@@ -1,7 +1,7 @@
 /*
- * irc-notify.c - notify lists for IRC plugin
+ * SPDX-FileCopyrightText: 2010-2025 Sébastien Helleu <flashcode@flashtux.org>
  *
- * Copyright (C) 2010-2024 Sébastien Helleu <flashcode@flashtux.org>
+ * SPDX-License-Identifier: GPL-3.0-or-later
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -19,6 +19,8 @@
  * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
+/* Notify lists for IRC plugin */
+
 #include <stdlib.h>
 #include <stddef.h>
 #include <stdio.h>
@@ -149,8 +151,7 @@ irc_notify_set_server_option (struct t_irc_server *server)
                 str2 = realloc (str, total_length);
                 if (!str2)
                 {
-                    if (str)
-                        free (str);
+                    free (str);
                     return;
                 }
                 str = str2;
@@ -275,43 +276,28 @@ irc_notify_build_message_with_nicks (struct t_irc_server *server,
                                      const char *separator,
                                      int *num_nicks)
 {
-    char *message, *message2;
-    int length, total_length, length_separator;
     struct t_irc_notify *ptr_notify;
+    char **message;
 
     *num_nicks = 0;
 
-    length = strlen (irc_message) + 1;
-    total_length = length;
-    length_separator = strlen (separator);
-
-    message = malloc (length);
+    message = weechat_string_dyn_alloc (256);
     if (!message)
         return NULL;
-    snprintf (message, length, "%s", irc_message);
+
+    weechat_string_dyn_concat (message, irc_message, -1);
 
     for (ptr_notify = server->notify_list; ptr_notify;
          ptr_notify = ptr_notify->next_notify)
     {
-        length = strlen (ptr_notify->nick);
-        total_length += length + length_separator;
-        message2 = realloc (message, total_length);
-        if (!message2)
-        {
-            if (message)
-                free (message);
-            message = NULL;
-            break;
-        }
-        message = message2;
         if (*num_nicks > 0)
-            strcat (message, separator);
-        strcat (message, ptr_notify->nick);
+            weechat_string_dyn_concat (message, separator, -1);
+        weechat_string_dyn_concat (message, ptr_notify->nick, -1);
 
         (*num_nicks)++;
     }
 
-    return message;
+    return weechat_string_dyn_free (message, 0);
 }
 
 /*
@@ -352,8 +338,7 @@ irc_notify_send_monitor (struct t_irc_server *server)
             weechat_hashtable_free (hashtable);
         }
     }
-    if (message)
-        free (message);
+    free (message);
 }
 
 /*
@@ -426,7 +411,7 @@ irc_notify_new_for_server (struct t_irc_server *server)
  */
 
 void
-irc_notify_new_for_all_servers ()
+irc_notify_new_for_all_servers (void)
 {
     struct t_irc_server *ptr_server;
 
@@ -455,7 +440,8 @@ irc_notify_free (struct t_irc_server *server, struct t_irc_notify *notify,
     if (notify->nick)
     {
         if ((server->monitor > 0) && remove_monitor
-            && (server->is_connected) && !irc_signal_upgrade_received)
+            && (server->is_connected)
+            && !weechat_irc_plugin->unload_with_upgrade)
         {
             /* remove one monitored nick */
             irc_server_sendf (notify->server,
@@ -464,8 +450,7 @@ irc_notify_free (struct t_irc_server *server, struct t_irc_notify *notify,
         }
         free (notify->nick);
     }
-    if (notify->away_message)
-        free (notify->away_message);
+    free (notify->away_message);
 
     /* remove notify from list */
     if (notify->prev_notify)
@@ -495,7 +480,7 @@ irc_notify_free_all (struct t_irc_server *server)
 {
     /* remove all monitored nicks */
     if ((server->monitor > 0) && (server->is_connected)
-        && !irc_signal_upgrade_received)
+        && !weechat_irc_plugin->unload_with_upgrade)
     {
         irc_server_sendf (server, IRC_SERVER_SEND_OUTQ_PRIO_LOW, NULL,
                           "MONITOR C");
@@ -649,26 +634,19 @@ irc_notify_send_signal (struct t_irc_notify *notify,
                         const char *away_message)
 {
     char signal[128], *data;
-    int length;
 
     snprintf (signal, sizeof (signal), "irc_notify_%s", type);
 
-    length = strlen (notify->server->name) + 1 + strlen (notify->nick) + 1
-        + ((away_message) ? strlen (away_message) : 0) + 1;
-    data = malloc (length);
-    if (data)
-    {
-        snprintf (data, length, "%s,%s%s%s",
-                  notify->server->name,
-                  notify->nick,
-                  (away_message && away_message[0]) ? "," : "",
-                  (away_message && away_message[0]) ? away_message : "");
-    }
+    weechat_asprintf (&data,
+                      "%s,%s%s%s",
+                      notify->server->name,
+                      notify->nick,
+                      (away_message && away_message[0]) ? "," : "",
+                      (away_message && away_message[0]) ? away_message : "");
 
     (void) weechat_hook_signal_send (signal, WEECHAT_HOOK_SIGNAL_STRING, data);
 
-    if (data)
-        free (data);
+    free (data);
 }
 
 /*
@@ -797,8 +775,7 @@ irc_notify_set_away_message (struct t_irc_notify *notify,
         irc_notify_send_signal (notify, "still_away", away_message);
     }
 
-    if (notify->away_message)
-        free (notify->away_message);
+    free (notify->away_message);
     notify->away_message = (away_message) ? strdup (away_message) : NULL;
 }
 
@@ -1037,10 +1014,8 @@ irc_notify_hsignal_cb (const void *pointer, void *data, const char *signal,
                             }
                         }
                     }
-                    if (irc_cmd)
-                        free (irc_cmd);
-                    if (arguments)
-                        free (arguments);
+                    free (irc_cmd);
+                    free (arguments);
                 }
             }
             if (!away_message_updated && !no_such_nick)
@@ -1106,8 +1081,7 @@ irc_notify_timer_ison_cb (const void *pointer, void *data, int remaining_calls)
                     weechat_hashtable_free (hashtable);
                 }
             }
-            if (message)
-                free (message);
+            free (message);
         }
     }
 
@@ -1242,15 +1216,15 @@ irc_notify_print_log (struct t_irc_server *server)
          ptr_notify = ptr_notify->next_notify)
     {
         weechat_log_printf ("");
-        weechat_log_printf ("  => notify (addr:0x%lx):", ptr_notify);
-        weechat_log_printf ("       server. . . . . . . : 0x%lx", ptr_notify->server);
-        weechat_log_printf ("       nick. . . . . . . . : '%s'",  ptr_notify->nick);
-        weechat_log_printf ("       check_away. . . . . : %d",    ptr_notify->check_away);
-        weechat_log_printf ("       is_on_server. . . . : %d",    ptr_notify->is_on_server);
-        weechat_log_printf ("       away_message. . . . : '%s'",  ptr_notify->away_message);
-        weechat_log_printf ("       ison_received . . . : %d",    ptr_notify->ison_received);
-        weechat_log_printf ("       prev_notify . . . . : 0x%lx", ptr_notify->prev_notify);
-        weechat_log_printf ("       next_notify . . . . : 0x%lx", ptr_notify->next_notify);
+        weechat_log_printf ("  => notify (addr:%p):", ptr_notify);
+        weechat_log_printf ("       server. . . . . . . : %p", ptr_notify->server);
+        weechat_log_printf ("       nick. . . . . . . . : '%s'", ptr_notify->nick);
+        weechat_log_printf ("       check_away. . . . . : %d", ptr_notify->check_away);
+        weechat_log_printf ("       is_on_server. . . . : %d", ptr_notify->is_on_server);
+        weechat_log_printf ("       away_message. . . . : '%s'", ptr_notify->away_message);
+        weechat_log_printf ("       ison_received . . . : %d", ptr_notify->ison_received);
+        weechat_log_printf ("       prev_notify . . . . : %p", ptr_notify->prev_notify);
+        weechat_log_printf ("       next_notify . . . . : %p", ptr_notify->next_notify);
     }
 }
 
@@ -1259,11 +1233,9 @@ irc_notify_print_log (struct t_irc_server *server)
  */
 
 void
-irc_notify_hook_timer_ison ()
+irc_notify_hook_timer_ison (void)
 {
-    if (irc_notify_timer_ison)
-        weechat_unhook (irc_notify_timer_ison);
-
+    weechat_unhook (irc_notify_timer_ison);
     irc_notify_timer_ison = weechat_hook_timer (
         60 * 1000 * weechat_config_integer (irc_config_network_notify_check_ison),
         0, 0, &irc_notify_timer_ison_cb, NULL, NULL);
@@ -1274,11 +1246,9 @@ irc_notify_hook_timer_ison ()
  */
 
 void
-irc_notify_hook_timer_whois ()
+irc_notify_hook_timer_whois (void)
 {
-    if (irc_notify_timer_whois)
-        weechat_unhook (irc_notify_timer_whois);
-
+    weechat_unhook (irc_notify_timer_whois);
     irc_notify_timer_whois = weechat_hook_timer (
         60 * 1000 * weechat_config_integer (irc_config_network_notify_check_whois),
         0, 0, &irc_notify_timer_whois_cb, NULL, NULL);
@@ -1289,7 +1259,7 @@ irc_notify_hook_timer_whois ()
  */
 
 void
-irc_notify_init ()
+irc_notify_init (void)
 {
     irc_notify_hook_timer_ison ();
     irc_notify_hook_timer_whois ();
@@ -1304,7 +1274,7 @@ irc_notify_init ()
  */
 
 void
-irc_notify_end ()
+irc_notify_end (void)
 {
     if (irc_notify_timer_ison)
     {
