@@ -1,7 +1,7 @@
 /*
- * relay-weechat-msg.c - build binary messages for WeeChat protocol
+ * SPDX-FileCopyrightText: 2003-2025 Sébastien Helleu <flashcode@flashtux.org>
  *
- * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
+ * SPDX-License-Identifier: GPL-3.0-or-later
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -19,6 +19,8 @@
  * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
+/* Build binary messages for WeeChat protocol */
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -175,6 +177,23 @@ relay_weechat_msg_add_long (struct t_relay_weechat_msg *msg, long value)
     relay_weechat_msg_add_bytes (msg, str_long, length);
 }
 
+/*
+ * Adds a long long integer to a message.
+ */
+
+void
+relay_weechat_msg_add_longlong (struct t_relay_weechat_msg *msg,
+                                long long value)
+{
+    char str_longlong[128];
+    unsigned char length;
+
+    snprintf (str_longlong, sizeof (str_longlong), "%lld", value);
+    length = strlen (str_longlong);
+    relay_weechat_msg_add_bytes (msg, &length, 1);
+    relay_weechat_msg_add_bytes (msg, str_longlong, length);
+}
+
 /*
  * Adds length + string to a message.
  */
@@ -283,6 +302,8 @@ relay_weechat_msg_hashtable_map_cb (void *data,
             relay_weechat_msg_add_pointer (msg, (void *)pointers[i]);
         else if (strcmp (type, WEECHAT_HASHTABLE_TIME) == 0)
             relay_weechat_msg_add_time (msg, *((time_t *)pointers[i]));
+        else if (strcmp (type, WEECHAT_HASHTABLE_LONGLONG) == 0)
+            relay_weechat_msg_add_longlong (msg, *((long long *)pointers[i]));
     }
 }
 
@@ -311,6 +332,8 @@ relay_weechat_msg_add_hashtable (struct t_relay_weechat_msg *msg,
             relay_weechat_msg_add_type (msg, RELAY_WEECHAT_MSG_OBJ_POINTER);
         else if (strcmp (type, WEECHAT_HASHTABLE_TIME) == 0)
             relay_weechat_msg_add_type (msg, RELAY_WEECHAT_MSG_OBJ_TIME);
+        else if (strcmp (type, WEECHAT_HASHTABLE_LONGLONG) == 0)
+            relay_weechat_msg_add_type (msg, RELAY_WEECHAT_MSG_OBJ_LONG);
     }
 
     /* number of items */
@@ -434,6 +457,7 @@ relay_weechat_msg_add_hdata_path (struct t_relay_weechat_msg *msg,
                                 relay_weechat_msg_add_type (msg, RELAY_WEECHAT_MSG_OBJ_INT);
                                 break;
                             case WEECHAT_HDATA_LONG:
+                            case WEECHAT_HDATA_LONGLONG:
                                 relay_weechat_msg_add_type (msg, RELAY_WEECHAT_MSG_OBJ_LONG);
                                 break;
                             case WEECHAT_HDATA_STRING:
@@ -480,6 +504,12 @@ relay_weechat_msg_add_hdata_path (struct t_relay_weechat_msg *msg,
                                                                                     pointer,
                                                                                     name));
                                     break;
+                                case WEECHAT_HDATA_LONGLONG:
+                                    relay_weechat_msg_add_longlong (msg,
+                                                                    weechat_hdata_longlong (hdata,
+                                                                                            pointer,
+                                                                                            name));
+                                    break;
                                 case WEECHAT_HDATA_STRING:
                                 case WEECHAT_HDATA_SHARED_STRING:
                                     relay_weechat_msg_add_string (msg,
@@ -694,6 +724,7 @@ relay_weechat_msg_add_hdata (struct t_relay_weechat_msg *msg,
                         strcat (keys_types, RELAY_WEECHAT_MSG_OBJ_INT);
                         break;
                     case WEECHAT_HDATA_LONG:
+                    case WEECHAT_HDATA_LONGLONG:
                         strcat (keys_types, RELAY_WEECHAT_MSG_OBJ_LONG);
                         break;
                     case WEECHAT_HDATA_STRING:
@@ -743,16 +774,11 @@ relay_weechat_msg_add_hdata (struct t_relay_weechat_msg *msg,
     rc = 1;
 
 end:
-    if (list_keys)
-        weechat_string_free_split (list_keys);
-    if (keys_types)
-        free (keys_types);
-    if (list_path)
-        weechat_string_free_split (list_path);
-    if (path_returned)
-        free (path_returned);
-    if (hdata_head)
-        free (hdata_head);
+    weechat_string_free_split (list_keys);
+    free (keys_types);
+    weechat_string_free_split (list_path);
+    free (path_returned);
+    free (hdata_head);
 
     return rc;
 }
@@ -1091,15 +1117,14 @@ relay_weechat_msg_compress_zlib (struct t_relay_client *client,
               msg->id);
 
     /* send compressed data */
-    relay_client_send (client, RELAY_CLIENT_MSG_STANDARD,
+    relay_client_send (client, RELAY_MSG_STANDARD,
                        (const char *)dest, dest_size + 5,
                        raw_message);
 
     rc = 1;
 
 error:
-    if (dest)
-        free (dest);
+    free (dest);
 
     return rc;
 }
@@ -1164,15 +1189,14 @@ relay_weechat_msg_compress_zstd (struct t_relay_client *client,
               msg->id);
 
     /* send compressed data */
-    relay_client_send (client, RELAY_CLIENT_MSG_STANDARD,
+    relay_client_send (client, RELAY_MSG_STANDARD,
                        (const char *)dest, comp_size + 5,
                        raw_message);
 
     rc = 1;
 
 error:
-    if (dest)
-        free (dest);
+    free (dest);
 
     return rc;
 #else
@@ -1225,7 +1249,7 @@ relay_weechat_msg_send (struct t_relay_client *client,
     /* send uncompressed data */
     snprintf (raw_message, sizeof (raw_message),
               "obj: %d bytes, id: %s", msg->data_size, msg->id);
-    relay_client_send (client, RELAY_CLIENT_MSG_STANDARD,
+    relay_client_send (client, RELAY_MSG_STANDARD,
                        msg->data, msg->data_size, raw_message);
 }
 
@@ -1239,10 +1263,8 @@ relay_weechat_msg_free (struct t_relay_weechat_msg *msg)
     if (!msg)
         return;
 
-    if (msg->id)
-        free (msg->id);
-    if (msg->data)
-        free (msg->data);
+    free (msg->id);
+    free (msg->data);
 
     free (msg);
 }
