@@ -1,7 +1,7 @@
 /*
- * test-relay-irc.cpp - test relay IRC protocol
+ * SPDX-FileCopyrightText: 2023-2025 Sébastien Helleu <flashcode@flashtux.org>
  *
- * Copyright (C) 2023-2024 Sébastien Helleu <flashcode@flashtux.org>
+ * SPDX-License-Identifier: GPL-3.0-or-later
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -19,9 +19,11 @@
  * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+/* Test relay IRC protocol */
+
 #include "CppUTest/TestHarness.h"
 
-#include "tests/tests.h"
+#include "tests.h"
 
 extern "C"
 {
@@ -85,7 +87,7 @@ extern struct t_arraylist *relay_irc_get_list_caps ();
              && (arraylist_size (sent_messages_client) > 0))            \
     {                                                                   \
         char **msg = test_build_error (                                 \
-            "Unexpected message(s) sent to the relay cleint",           \
+            "Unexpected message(s) sent to the relay client",           \
             NULL,                                                       \
             NULL,                                                       \
             NULL,                                                       \
@@ -793,14 +795,14 @@ TEST(RelayIrcWithClient, RecvCommandCapab)
     LONGS_EQUAL(0, RELAY_IRC_DATA(ptr_relay_client, server_capabilities));
     LONGS_EQUAL(0, RELAY_IRC_DATA(ptr_relay_client, cap_end_received));
 
-    /* request with empty list: end of capability negociation */
+    /* request with empty list: end of capability negotiation */
     CLIENT_RECV(":alice!user@host CAP REQ :");
     CHECK_SENT_CLIENT(":weechat.relay.irc CAP nick NAK :");
     LONGS_EQUAL(1, RELAY_IRC_DATA(ptr_relay_client, cap_end_received));
 
     RELAY_IRC_DATA(ptr_relay_client, cap_end_received) = 0;
 
-    /* end capability negociation */
+    /* end capability negotiation */
     CLIENT_RECV(":alice!user@host CAP END");
     LONGS_EQUAL(1, RELAY_IRC_DATA(ptr_relay_client, cap_end_received));
 }
@@ -820,12 +822,12 @@ TEST(RelayIrcWithClient, ParseCtcp)
     ctcp_type = (char *)0x01;
     ctcp_params = (char *)0x01;
     relay_irc_parse_ctcp (NULL, &ctcp_type, &ctcp_params);
-    POINTERS_EQUAL(NULL, ctcp_type);
-    POINTERS_EQUAL(NULL, ctcp_params);
+    STRCMP_EQUAL(NULL, ctcp_type);
+    STRCMP_EQUAL(NULL, ctcp_params);
 
     ctcp_type = (char *)0x01;
     ctcp_params = (char *)0x01;
-    relay_irc_parse_ctcp ("\01ACTION is testing\01", &ctcp_type, &ctcp_params);
+    relay_irc_parse_ctcp ("\001ACTION is testing\001", &ctcp_type, &ctcp_params);
     STRCMP_EQUAL("ACTION", ctcp_type);
     STRCMP_EQUAL("is testing", ctcp_params);
     free (ctcp_type);
@@ -833,7 +835,7 @@ TEST(RelayIrcWithClient, ParseCtcp)
 
     ctcp_type = (char *)0x01;
     ctcp_params = (char *)0x01;
-    relay_irc_parse_ctcp ("\01ACTION   is testing  \01 extra", &ctcp_type, &ctcp_params);
+    relay_irc_parse_ctcp ("\001ACTION   is testing  \001 extra", &ctcp_type, &ctcp_params);
     STRCMP_EQUAL("ACTION", ctcp_type);
     STRCMP_EQUAL("  is testing  ", ctcp_params);
     free (ctcp_type);
@@ -841,34 +843,34 @@ TEST(RelayIrcWithClient, ParseCtcp)
 
     ctcp_type = (char *)0x01;
     ctcp_params = (char *)0x01;
-    relay_irc_parse_ctcp ("\01VERSION\01", &ctcp_type, &ctcp_params);
+    relay_irc_parse_ctcp ("\001VERSION\001", &ctcp_type, &ctcp_params);
     STRCMP_EQUAL("VERSION", ctcp_type);
-    POINTERS_EQUAL(NULL, ctcp_params);
+    STRCMP_EQUAL(NULL, ctcp_params);
     free (ctcp_type);
 
     ctcp_type = (char *)0x01;
     ctcp_params = (char *)0x01;
-    relay_irc_parse_ctcp ("\01ACTION is testing", &ctcp_type, &ctcp_params);
-    POINTERS_EQUAL(NULL, ctcp_type);
-    POINTERS_EQUAL(NULL, ctcp_params);
+    relay_irc_parse_ctcp ("\001ACTION is testing", &ctcp_type, &ctcp_params);
+    STRCMP_EQUAL(NULL, ctcp_type);
+    STRCMP_EQUAL(NULL, ctcp_params);
 
     ctcp_type = (char *)0x01;
     ctcp_params = (char *)0x01;
-    relay_irc_parse_ctcp ("\01VERSION", &ctcp_type, &ctcp_params);
-    POINTERS_EQUAL(NULL, ctcp_type);
-    POINTERS_EQUAL(NULL, ctcp_params);
+    relay_irc_parse_ctcp ("\001VERSION", &ctcp_type, &ctcp_params);
+    STRCMP_EQUAL(NULL, ctcp_type);
+    STRCMP_EQUAL(NULL, ctcp_params);
 
     ctcp_type = (char *)0x01;
     ctcp_params = (char *)0x01;
     relay_irc_parse_ctcp ("test", &ctcp_type, &ctcp_params);
-    POINTERS_EQUAL(NULL, ctcp_type);
-    POINTERS_EQUAL(NULL, ctcp_params);
+    STRCMP_EQUAL(NULL, ctcp_type);
+    STRCMP_EQUAL(NULL, ctcp_params);
 
     ctcp_type = (char *)0x01;
     ctcp_params = (char *)0x01;
     relay_irc_parse_ctcp ("", &ctcp_type, &ctcp_params);
-    POINTERS_EQUAL(NULL, ctcp_type);
-    POINTERS_EQUAL(NULL, ctcp_params);
+    STRCMP_EQUAL(NULL, ctcp_type);
+    STRCMP_EQUAL(NULL, ctcp_params);
 }
 
 /*
